@@ -7,27 +7,29 @@ special Python builtins module.
 import os
 import re
 import sys
-import stat
 import time
 import shlex
 import atexit
 import signal
 import inspect
 import builtins
+import tempfile
 from glob import glob, iglob
-from subprocess import Popen, PIPE, STDOUT
+from subprocess import Popen, PIPE, STDOUT, CalledProcessError
 from contextlib import contextmanager
 from collections import Sequence, MutableMapping, Iterable
 
 from xonsh.tools import (
     suggest_commands, XonshError, ON_POSIX, ON_WINDOWS, string_types,
-    expandvars,
+    expandvars, CommandsCache
 )
 from xonsh.inspectors import Inspector
 from xonsh.environ import Env, default_env, locate_binary
 from xonsh.aliases import DEFAULT_ALIASES
 from xonsh.jobs import add_job, wait_for_active_job
-from xonsh.proc import ProcProxy, SimpleProcProxy, TeePTYProc
+from xonsh.proc import (ProcProxy, SimpleProcProxy, ForegroundProcProxy,
+                        SimpleForegroundProcProxy, TeePTYProc,
+                        CompletedCommand, HiddenCompletedCommand)
 from xonsh.history import History
 from xonsh.foreign_shells import load_foreign_aliases
 
@@ -45,6 +47,7 @@ def resetting_signal_handle(sig, f):
     once the new handle is finished.
     """
     oldh = signal.getsignal(sig)
+
     def newh(s=None, frame=None):
         f(s, frame)
         signal.signal(sig, oldh)
@@ -185,12 +188,23 @@ def expand_path(s):
     return os.path.expanduser(s)
 
 
+WINDOWS_DRIVE_MATCHER = re.compile(r'^\w:')
+
+
 def expand_case_matching(s):
     """Expands a string to a case insenstive globable string."""
     t = []
     openers = {'[', '{'}
     closers = {']', '}'}
     nesting = 0
+
+    drive_part = WINDOWS_DRIVE_MATCHER.match(s) if ON_WINDOWS else None
+
+    if drive_part:
+        drive_part = drive_part.group(0)
+        t.append(drive_part)
+        s = s[len(drive_part):]
+
     for c in s:
         if c in openers:
             nesting += 1
@@ -211,8 +225,7 @@ def expand_case_matching(s):
                                              c)
                 c = newc
         t.append(c)
-    t = ''.join(t)
-    return t
+    return ''.join(t)
 
 
 def reglob(path, parts=None, i=None):
@@ -243,12 +256,12 @@ def reglob(path, parts=None, i=None):
     if i1 == len(parts):
         for f in files:
             p = os.path.join(base, f)
-            if regex.match(p) is not None:
+            if regex.fullmatch(p) is not None:
                 paths.append(p)
     else:
         for f in files:
             p = os.path.join(base, f)
-            if regex.match(p) is None or not os.path.isdir(p):
+            if regex.fullmatch(p) is None or not os.path.isdir(p):
                 continue
             paths += reglob(p, parts=parts, i=i1)
     return paths
@@ -259,7 +272,8 @@ def regexpath(s):
     paths that match the regex.
     """
     s = expand_path(s)
-    return reglob(s)
+    o = reglob(s)
+    return o if len(o) != 0 else [s]
 
 
 def globpath(s, ignore_case=False):
@@ -282,68 +296,6 @@ def iglobpath(s, ignore_case=False):
 RE_SHEBANG = re.compile(r'#![ \t]*(.+?)$')
 
 
-def _is_executable_file(path):
-    """Checks that path is an executable regular file (or a symlink to a file).
-
-    This is roughly ``os.path isfile(path) and os.access(path, os.X_OK)``, but
-    on some platforms :func:`os.access` gives us the wrong answer, so this
-    checks permission bits directly.
-    """
-    # follow symlinks,
-    fpath = os.path.realpath(path)
-
-    # return False for non-files (directories, fifo, etc.)
-    if not os.path.isfile(fpath):
-        return False
-
-    # On Solaris, etc., "If the process has appropriate privileges, an
-    # implementation may indicate success for X_OK even if none of the
-    # execute file permission bits are set."
-    #
-    # For this reason, it is necessary to explicitly check st_mode
-
-    # get file mode using os.stat, and check if `other',
-    # that is anybody, may read and execute.
-    mode = os.stat(fpath).st_mode
-    if mode & stat.S_IROTH and mode & stat.S_IXOTH:
-        return True
-
-    # get current user's group ids, and check if `group',
-    # when matching ours, may read and execute.
-    user_gids = os.getgroups() + [os.getgid()]
-    if (os.stat(fpath).st_gid in user_gids and
-            mode & stat.S_IRGRP and mode & stat.S_IXGRP):
-        return True
-
-    # finally, if file owner matches our effective userid,
-    # check if `user', may read and execute.
-    user_gids = os.getgroups() + [os.getgid()]
-    if (os.stat(fpath).st_uid == os.geteuid() and
-            mode & stat.S_IRUSR and mode & stat.S_IXUSR):
-        return True
-
-    return False
-
-
-def _get_runnable_name(fname):
-    if os.path.isfile(fname) and fname != os.path.basename(fname):
-        return fname
-    for d in builtins.__xonsh_env__.get('PATH'):
-        if os.path.isdir(d):
-            files = os.listdir(d)
-            if ON_WINDOWS:
-                PATHEXT = builtins.__xonsh_env__.get('PATHEXT')
-                for dirfile in files:
-                    froot, ext = os.path.splitext(dirfile)
-                    if fname == froot and ext.upper() in PATHEXT:
-                        return os.path.join(d, dirfile)
-            if fname in files:
-                path = os.path.join(d, fname)
-                if _is_executable_file(path):
-                    return path
-    return None
-
-
 def _is_binary(fname, limit=80):
     with open(fname, 'rb') as f:
         for i in range(limit):
@@ -380,10 +332,10 @@ def get_script_subproc_command(fname, args):
         raise PermissionError
 
     if ON_POSIX and not os.access(fname, os.R_OK):
-        # on some systems, some importnat programs (e.g. sudo) will have execute
-        # permissions but not read/write permisions. This enables things with the SUID
-        # set to be run. Needs to come before _is_binary() is called, because that
-        # function tries to read the file.
+        # on some systems, some importnat programs (e.g. sudo) will have
+        # execute permissions but not read/write permisions. This enables
+        # things with the SUID set to be run. Needs to come before _is_binary()
+        # is called, because that function tries to read the file.
         return [fname] + args
     elif _is_binary(fname):
         # if the file is a binary, we should call it directly
@@ -457,7 +409,7 @@ def _redirect_io(streams, r, loc=None):
     if r.replace('&', '') in _E2O_MAP:
         if 'stderr' in streams:
             raise XonshError('Multiple redirects for stderr')
-        streams['stderr'] = STDOUT
+        streams['stderr'] = ('<stdout>', 'a', STDOUT)
         return
 
     orig, mode, dest = _REDIR_REGEX.match(r).groups()
@@ -484,7 +436,7 @@ def _redirect_io(streams, r, loc=None):
         elif 'stdin' in streams:
             raise XonshError('Multiple inputs for stdin')
         else:
-            streams['stdin'] = _open(loc, mode)
+            streams['stdin'] = (loc, 'r', _open(loc, mode))
     elif mode in _WRITE_MODES:
         if orig in _REDIR_ALL:
             if 'stderr' in streams:
@@ -514,13 +466,13 @@ def _redirect_io(streams, r, loc=None):
 
         f = _open(loc, mode)
         for t in targets:
-            streams[t] = f
+            streams[t] = (loc, mode, f)
 
     else:
         raise XonshError('Unrecognized redirection command: {}'.format(r))
 
 
-def run_subproc(cmds, captured=True):
+def run_subproc(cmds, captured=False):
     """Runs a subprocess, in its many forms. This takes a list of 'commands,'
     which may be a list of command line arguments or a string, representing
     a special connecting character.  For example::
@@ -535,6 +487,7 @@ def run_subproc(cmds, captured=True):
     """
     global ENV
     background = False
+    procinfo = {}
     if cmds[-1] == '&':
         background = True
         cmds = cmds[:-1]
@@ -542,7 +495,10 @@ def run_subproc(cmds, captured=True):
     last_cmd = len(cmds) - 1
     procs = []
     prev_proc = None
+    _capture_streams = captured in {'stdout', 'object'}
     for ix, cmd in enumerate(cmds):
+        starttime = time.time()
+        procinfo['args'] = list(cmd)
         stdin = None
         stderr = None
         if isinstance(cmd, string_types):
@@ -564,15 +520,17 @@ def run_subproc(cmds, captured=True):
         if 'stdin' in streams:
             if prev_proc is not None:
                 raise XonshError('Multiple inputs for stdin')
-            stdin = streams['stdin']
+            stdin = streams['stdin'][-1]
+            procinfo['stdin_redirect'] = streams['stdin'][:-1]
         elif prev_proc is not None:
             stdin = prev_proc.stdout
         # set standard output
         if 'stdout' in streams:
             if ix != last_cmd:
                 raise XonshError('Multiple redirects for stdout')
-            stdout = streams['stdout']
-        elif captured or ix != last_cmd:
+            stdout = streams['stdout'][-1]
+            procinfo['stdout_redirect'] = streams['stdout'][:-1]
+        elif _capture_streams or ix != last_cmd:
             stdout = PIPE
         elif builtins.__xonsh_stdout_uncaptured__ is not None:
             stdout = builtins.__xonsh_stdout_uncaptured__
@@ -580,16 +538,20 @@ def run_subproc(cmds, captured=True):
             stdout = None
         # set standard error
         if 'stderr' in streams:
-            stderr = streams['stderr']
+            stderr = streams['stderr'][-1]
+            procinfo['stderr_redirect'] = streams['stderr'][:-1]
+        elif captured == 'object':
+            stderr = PIPE
         elif builtins.__xonsh_stderr_uncaptured__ is not None:
             stderr = builtins.__xonsh_stderr_uncaptured__
-        uninew = (ix == last_cmd) and (not captured)
+        uninew = (ix == last_cmd) and (not _capture_streams)
         alias = builtins.aliases.get(cmd[0], None)
-        if (alias is None
-            and builtins.__xonsh_env__.get('AUTO_CD')
-            and len(cmd) == 1
-            and os.path.isdir(cmd[0])
-            and locate_binary(cmd[0], cwd=None) is None):
+        procinfo['alias'] = alias
+        if (alias is None and
+                builtins.__xonsh_env__.get('AUTO_CD') and
+                len(cmd) == 1 and
+                os.path.isdir(cmd[0]) and
+                locate_binary(cmd[0]) is None):
             cmd.insert(0, 'cd')
             alias = builtins.aliases.get('cd', None)
 
@@ -598,7 +560,7 @@ def run_subproc(cmds, captured=True):
         else:
             if alias is not None:
                 cmd = alias + cmd[1:]
-            n = _get_runnable_name(cmd[0])
+            n = locate_binary(cmd[0])
             if n is None:
                 aliased_cmd = cmd
             else:
@@ -607,13 +569,28 @@ def run_subproc(cmds, captured=True):
                 except PermissionError:
                     e = 'xonsh: subprocess mode: permission denied: {0}'
                     raise XonshError(e.format(cmd[0]))
+        _stdin_file = None
+        if (stdin is not None and
+                ENV.get('XONSH_STORE_STDIN') and
+                captured == 'object' and
+                'cat' in __xonsh_commands_cache__ and
+                'tee' in __xonsh_commands_cache__):
+            _stdin_file = tempfile.NamedTemporaryFile()
+            cproc = Popen(['cat'],
+                          stdin=stdin,
+                          stdout=PIPE)
+            tproc = Popen(['tee', _stdin_file.name],
+                          stdin=cproc.stdout,
+                          stdout=PIPE)
+            stdin = tproc.stdout
         if callable(aliased_cmd):
             prev_is_proxy = True
+            bgable = getattr(aliased_cmd, '__xonsh_backgroundable__', True)
             numargs = len(inspect.signature(aliased_cmd).parameters)
             if numargs == 2:
-                cls = SimpleProcProxy
+                cls = SimpleProcProxy if bgable else SimpleForegroundProcProxy
             elif numargs == 4:
-                cls = ProcProxy
+                cls = ProcProxy if bgable else ForegroundProcProxy
             else:
                 e = 'Expected callable with 2 or 4 arguments, not {}'
                 raise XonshError(e.format(numargs))
@@ -622,8 +599,9 @@ def run_subproc(cmds, captured=True):
                        universal_newlines=uninew)
         else:
             prev_is_proxy = False
-            usetee = (stdout is None) and (not background) and \
-                     ENV.get('XONSH_STORE_STDOUT', False)
+            usetee = ((stdout is None) and
+                      (not background) and
+                      ENV.get('XONSH_STORE_STDOUT', False))
             cls = TeePTYProc if usetee else Popen
             subproc_kwargs = {}
             if ON_POSIX and cls is Popen:
@@ -660,7 +638,9 @@ def run_subproc(cmds, captured=True):
             'obj': prev_proc,
             'bg': background
         })
-    if ENV.get('XONSH_INTERACTIVE') and not ENV.get('XONSH_STORE_STDOUT'):
+    if (ENV.get('XONSH_INTERACTIVE') and
+            not ENV.get('XONSH_STORE_STDOUT') and
+            not _capture_streams):
         # set title here to get current command running
         try:
             builtins.__xonsh_shell__.settitle()
@@ -673,27 +653,70 @@ def run_subproc(cmds, captured=True):
     wait_for_active_job()
     hist = builtins.__xonsh_history__
     hist.last_cmd_rtn = prev_proc.returncode
+    # get output
+    output = b''
     if write_target is None:
-        # get output
-        output = b''
         if prev_proc.stdout not in (None, sys.stdout):
             output = prev_proc.stdout.read()
-        if captured:
+        if _capture_streams:
             # to get proper encoding from Popen, we have to
             # use a byte stream and then implement universal_newlines here
             output = output.decode(encoding=ENV.get('XONSH_ENCODING'),
                                    errors=ENV.get('XONSH_ENCODING_ERRORS'))
             output = output.replace('\r\n', '\n')
-            return output
         else:
             hist.last_cmd_out = output
+        if (captured == 'object' and
+                prev_proc.stderr not in {None, sys.stderr}):
+            errout = prev_proc.stderr.read()
+            errout = errout.decode(encoding=ENV.get('XONSH_ENCODING'),
+                                   errors=ENV.get('XONSH_ENCODING_ERRORS'))
+            errout = errout.replace('\r\n', '\n')
+            procinfo['stderr'] = errout
+
+    if (not prev_is_proxy and
+            hist.last_cmd_rtn is not None and
+            hist.last_cmd_rtn > 0 and
+            ENV.get('RAISE_SUBPROC_ERROR')):
+        raise CalledProcessError(hist.last_cmd_rtn, aliased_cmd, output=output)
+    if captured == 'stdout':
+        return output
+    elif captured is not False:
+        procinfo['pid'] = prev_proc.pid
+        procinfo['returncode'] = prev_proc.returncode
+        procinfo['timestamp'] = (starttime, time.time())
+        if captured == 'object':
+            procinfo['stdout'] = output
+            if _stdin_file is not None:
+                _stdin_file.seek(0)
+                procinfo['stdin'] = _stdin_file.read().decode()
+                _stdin_file.close()
+            return CompletedCommand(**procinfo)
+        else:
+            return HiddenCompletedCommand(**procinfo)
 
 
-def subproc_captured(*cmds):
+def subproc_captured_stdout(*cmds):
     """Runs a subprocess, capturing the output. Returns the stdout
     that was produced as a str.
     """
-    return run_subproc(cmds, captured=True)
+    return run_subproc(cmds, captured='stdout')
+
+
+def subproc_captured_object(*cmds):
+    """
+    Runs a subprocess, capturing the output. Returns an instance of
+    ``CompletedCommand`` representing the completed command.
+    """
+    return run_subproc(cmds, captured='object')
+
+
+def subproc_captured_hiddenobject(*cmds):
+    """
+    Runs a subprocess, capturing the output. Returns an instance of
+    ``HiddenCompletedCommand`` representing the completed command.
+    """
+    return run_subproc(cmds, captured='hiddenobject')
 
 
 def subproc_uncaptured(*cmds):
@@ -736,9 +759,12 @@ def load_builtins(execer=None, config=None):
     if hasattr(builtins, 'quit'):
         builtins.__xonsh_pyquit__ = builtins.quit
         del builtins.quit
-    builtins.__xonsh_subproc_captured__ = subproc_captured
+    builtins.__xonsh_subproc_captured_stdout__ = subproc_captured_stdout
+    builtins.__xonsh_subproc_captured_object__ = subproc_captured_object
+    builtins.__xonsh_subproc_captured_hiddenobject__ = subproc_captured_hiddenobject
     builtins.__xonsh_subproc_uncaptured__ = subproc_uncaptured
     builtins.__xonsh_execer__ = execer
+    builtins.__xonsh_commands_cache__ = CommandsCache()
     builtins.__xonsh_all_jobs__ = {}
     builtins.__xonsh_active_job__ = None
     builtins.__xonsh_ensure_list_of_strs__ = ensure_list_of_strs
@@ -750,15 +776,18 @@ def load_builtins(execer=None, config=None):
     builtins.aliases.update(load_foreign_aliases(issue_warning=False))
     # history needs to be started after env and aliases
     # would be nice to actually include non-detyped versions.
-    builtins.__xonsh_history__ = History(env=ENV.detype(), #aliases=builtins.aliases,
+    builtins.__xonsh_history__ = History(env=ENV.detype(),
                                          ts=[time.time(), None], locked=True)
-    lastflush = lambda s=None, f=None: builtins.__xonsh_history__.flush(at_exit=True)
-    atexit.register(lastflush)
+    atexit.register(_lastflush)
     for sig in AT_EXIT_SIGNALS:
-        resetting_signal_handle(sig, lastflush)
+        resetting_signal_handle(sig, _lastflush)
     BUILTINS_LOADED = True
 
 
+def _lastflush(s=None, f=None):
+    builtins.__xonsh_history__.flush(at_exit=True)
+
+
 def unload_builtins():
     """Removes the xonsh builtins from the Python builtins, if the
     BUILTINS_LOADED is True, sets BUILTINS_LOADED to False, and returns.
@@ -785,9 +814,12 @@ def unload_builtins():
              '__xonsh_stderr_uncaptured__',
              '__xonsh_pyexit__',
              '__xonsh_pyquit__',
-             '__xonsh_subproc_captured__',
+             '__xonsh_subproc_captured_stdout__',
+             '__xonsh_subproc_captured_object__',
+             '__xonsh_subproc_captured_hiddenobject__',
              '__xonsh_subproc_uncaptured__',
              '__xonsh_execer__',
+             '__xonsh_commands_cache__',
              'evalx',
              'execx',
              'compilex',
