@@ -12,7 +12,8 @@ import importlib
 import subprocess
 
 from xonsh.built_ins import iglobpath, expand_path
-from xonsh.tools import subexpr_from_unbalanced, get_sep, check_for_partial_string, RE_STRING_START
+from xonsh.tools import (subexpr_from_unbalanced, get_sep,
+                         check_for_partial_string, RE_STRING_START)
 from xonsh.tools import ON_WINDOWS
 
 
@@ -124,10 +125,7 @@ class Completer(object):
 
     def __init__(self):
         # initialize command cache
-        self._path_checksum = None
-        self._alias_checksum = None
-        self._path_mtime = -1
-        self._cmds_cache = frozenset()
+        self._cmds_cache = builtins.__xonsh_commands_cache__
         self._man_completer = ManCompleter()
         try:
             # FIXME this could be threaded for faster startup times
@@ -163,8 +161,6 @@ class Completer(object):
             (only used with prompt_toolkit)
         """
         space = ' '  # intern some strings for faster appending
-        slash = '/'
-        dot = '.'
         ctx = ctx or {}
         prefixlow = prefix.lower()
         _line = line
@@ -222,7 +218,7 @@ class Completer(object):
             if cmd == 'import' and begidx == len('import '):
                 # completing module to import
                 return sorted(self.module_complete(prefix)), lprefix
-            if cmd in self._all_commands():
+            if cmd in self._cmds_cache:
                 # subproc mode; do path completions
                 comps = self.path_complete(prefix, path_str_start,
                                            path_str_end, cdpath=True)
@@ -338,7 +334,7 @@ class Completer(object):
         csc = builtins.__xonsh_env__.get('CASE_SENSITIVE_COMPLETIONS')
         startswither = startswithnorm if csc else startswithlow
         return {s + space
-                for s in self._all_commands()
+                for s in self._cmds_cache
                 if startswither(s, cmd, cmdlow)}
 
     def module_complete(self, prefix):
@@ -393,7 +389,6 @@ class Completer(object):
 
     def path_complete(self, prefix, start, end, cdpath=False):
         """Completes based on a path name."""
-        space = ' '  # intern some strings for faster appending
         tilde = '~'
         paths = set()
         csc = builtins.__xonsh_env__.get('CASE_SENSITIVE_COMPLETIONS')
@@ -431,19 +426,13 @@ class Completer(object):
             prefix = shlex.quote(prefix)
 
         script = BASH_COMPLETE_SCRIPT.format(filename=fnme,
-                                             line=' '.join(shlex.quote(p) for p in splt),
-                                             comp_line=shlex.quote(line),
-                                             n=n,
-                                             func=func,
-                                             cmd=cmd,
-                                             end=endidx + 1,
-                                             prefix=prefix,
-                                             prev=shlex.quote(prev))
+                    line=' '.join(shlex.quote(p) for p in splt),
+                    comp_line=shlex.quote(line), n=n, func=func, cmd=cmd,
+                    end=endidx + 1, prefix=prefix, prev=shlex.quote(prev))
         try:
-            out = subprocess.check_output(['bash'],
-                                          input=script,
-                                          universal_newlines=True,
-                                          stderr=subprocess.PIPE)
+            out = subprocess.check_output(['bash'], input=script,
+                    universal_newlines=True, stderr=subprocess.PIPE,
+                    env=builtins.__xonsh_env__.detype())
         except subprocess.CalledProcessError:
             out = ''
 
@@ -467,7 +456,7 @@ class Completer(object):
             return
         inp.append('complete -p\n')
         out = subprocess.check_output(['bash'], input='\n'.join(inp),
-                                      universal_newlines=True)
+                env=builtins.__xonsh_env__.detype(), universal_newlines=True)
         for line in out.splitlines():
             head, cmd = line.rsplit(' ', 1)
             if len(cmd) == 0 or cmd == 'cd':
@@ -488,7 +477,7 @@ class Completer(object):
             inp.append('declare -F ' + ' '.join([f for f in bash_funcs]))
             inp.append('shopt -u extdebug\n')
         out = subprocess.check_output(['bash'], input='\n'.join(inp),
-                                      universal_newlines=True)
+                env=builtins.__xonsh_env__.detype(), universal_newlines=True)
         func_files = {}
         for line in out.splitlines():
             parts = line.split()
@@ -545,33 +534,6 @@ class Completer(object):
             attrs.add(comp)
         return attrs
 
-    def _all_commands(self):
-        path = builtins.__xonsh_env__.get('PATH', [])
-        # did PATH change?
-        path_hash = hash(frozenset(path))
-        cache_valid = path_hash == self._path_checksum
-        self._path_checksum = path_hash
-        # did aliases change?
-        al_hash = hash(frozenset(builtins.aliases.keys()))
-        cache_valid = cache_valid and al_hash == self._alias_checksum
-        self._alias_checksum = al_hash
-        pm = self._path_mtime
-        # did the contents of any directory in PATH change?
-        for d in filter(os.path.isdir, path):
-            m = os.stat(d).st_mtime
-            if m > pm:
-                pm = m
-                cache_valid = False
-        self._path_mtime = pm
-        if cache_valid:
-            return self._cmds_cache
-        allcmds = set()
-        for d in filter(os.path.isdir, path):
-            allcmds |= set(os.listdir(d))
-        allcmds |= set(builtins.aliases.keys())
-        self._cmds_cache = frozenset(allcmds)
-        return self._cmds_cache
-
 
 OPTIONS_PATH = os.path.expanduser('~') + "/.xonsh_man_completions"
 SCRAPE_RE = re.compile(r'^(?:\s*(?:-\w|--[a-z0-9-]+)[\s,])+', re.M)
@@ -597,11 +559,10 @@ class ManCompleter(object):
         if cmd not in self._options.keys():
             try:
                 manpage = subprocess.Popen(["man", cmd],
-                                           stdout=subprocess.PIPE,
-                                           stderr=subprocess.DEVNULL)
+                            stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
                 # This is a trick to get rid of reverse line feeds
                 text = subprocess.check_output(["col", "-b"],
-                                               stdin=manpage.stdout)
+                        stdin=manpage.stdout)
                 text = text.decode('utf-8')
                 scraped_text = ' '.join(SCRAPE_RE.findall(text))
                 matches = INNER_OPTIONS_RE.findall(scraped_text)
