@@ -12,8 +12,6 @@ from ply.lex import LexToken
 
 from xonsh.tools import VER_3_5, VER_MAJOR_MINOR
 
-future_kwlist = []
-
 token_map = {}
 """
 Mapping from ``tokenize`` tokens (or token types) to PLY token types.  If a
@@ -51,8 +49,6 @@ token_map[tokenize.DEDENT] = 'DEDENT'
 if VER_3_5 <= VER_MAJOR_MINOR:
     token_map[tokenize.ASYNC] = 'ASYNC'
     token_map[tokenize.AWAIT] = 'AWAIT'
-else:
-    future_kwlist += ['async', 'await']
 
 _REDIRECT_NAMES = frozenset({'out', 'err', 'all', 'o', 'e', 'a'})
 
@@ -222,6 +218,26 @@ def handle_question(state, token, stream):
             yield from handle_token(state, n, stream)
 
 
+def handle_bang(state, token, stream):
+    """
+    Function for generating PLY tokens starting with !
+    """
+    n = next(stream, None)
+    if (n is not None and n.type == tokenize.OP
+            and n.string == '(' and n.start == token.end):
+        state['pymode'].append((False, '!(', ')', token.start))
+        state['last'] = n
+        yield _new_token('BANG_LPAREN', '!(', token.start)
+    elif (n is not None and n.type == tokenize.OP
+            and n.string == '[' and n.start == token.end):
+        state['pymode'].append((False, '![', ']', token.start))
+        state['last'] = n
+        yield _new_token('BANG_LBRACKET', '![', token.start)
+    else:
+        e = 'unexpected single token: !'
+        yield _new_token("ERRORTOKEN", e, token.start)
+
+
 def handle_backtick(state, token, stream):
     """
     Function for generating PLY tokens representing regex globs.
@@ -373,6 +389,7 @@ special_handlers = {
     (tokenize.ERRORTOKEN, '$'): handle_dollar,
     (tokenize.ERRORTOKEN, '`'): handle_backtick,
     (tokenize.ERRORTOKEN, '?'): handle_question,
+    (tokenize.ERRORTOKEN, '!'): handle_bang,
     (tokenize.ERRORTOKEN, ' '): handle_error_space,
 }
 """
@@ -478,6 +495,7 @@ class Lexer(object):
         """
         self.fname = ''
         self.last = None
+        self.beforelast = None
 
     def build(self, **kwargs):
         """Part of the PLY lexer API."""
@@ -492,6 +510,7 @@ class Lexer(object):
 
     def token(self):
         """Retrieves the next token."""
+        self.beforelast = self.last
         self.last = next(self.token_stream, None)
         return self.last
 
@@ -519,7 +538,9 @@ class Lexer(object):
         'DOUBLE_QUESTION',       # ??
         'AT_LPAREN',             # @(
         'DOLLAR_NAME',           # $NAME
+        'BANG_LPAREN',           # !(
+        'BANG_LBRACKET',         # ![
         'DOLLAR_LPAREN',         # $(
         'DOLLAR_LBRACE',         # ${
         'DOLLAR_LBRACKET',       # $[
-    ) + tuple(i.upper() for i in kwlist) + tuple(i.upper() for i in future_kwlist)
+    ) + tuple(i.upper() for i in kwlist)
