@@ -4,6 +4,7 @@ import builtins
 
 from prompt_toolkit.filters import Filter, IsMultiline
 from prompt_toolkit.keys import Keys
+from xonsh.tools import ON_WINDOWS
 
 env = builtins.__xonsh_env__
 indent_ = env.get('INDENT')
@@ -35,13 +36,16 @@ def carriage_return(b, cli):
         b.insert_text(indent_, fire_event=False)
     # if current line isn't blank, check dedent tokens
     elif (not current_line_blank and
-            b.document.current_line.split(maxsplit=1)[0] in DEDENT_TOKENS):
+            b.document.current_line.split(maxsplit=1)[0] in DEDENT_TOKENS and
+            b.document.line_count > 1):
         b.newline(copy_margin=True)
         _ = b.delete_before_cursor(count=len(indent_))
     elif (not b.document.on_first_line and
             not current_line_blank):
         b.newline(copy_margin=True)
-    elif b.document.char_before_cursor == '\\':
+    elif (b.document.char_before_cursor == '\\' and 
+            not (not builtins.__xonsh_env__.get('FORCE_POSIX_PATHS') 
+                and ON_WINDOWS)):
         b.newline()
     elif (b.document.find_next_word_beginning() is not None and
             (any(not _is_blank(i)
@@ -69,7 +73,8 @@ class TabShouldInsertIndentFilter(Filter):
 
 def can_compile(src):
     """Returns whether the code can be compiled, i.e. it is valid xonsh."""
-    src = src if src.endswith('\n') else '{}\n'.format(src)
+    src = src if src.endswith('\n') else src + '\n'
+    src = src.lstrip()
     try:
         builtins.__xonsh_execer__.compile(src, mode='single', glbs=None,
                                           locs=builtins.__xonsh_ctx__)
