@@ -30,6 +30,19 @@ from contextlib import contextmanager
 from collections import OrderedDict, Sequence, Set
 from warnings import warn
 
+#
+# Check pygments
+#
+
+def pygments_version():
+    """Returns the Pygments version or False."""
+    try:
+        import pygments
+        v = pygments.__version__
+    except ImportError:
+        v = False
+    return v
+
 if sys.version_info[0] >= 3:
     string_types = (str, bytes)
     unicode_type = str
@@ -37,6 +50,12 @@ else:
     string_types = (str, unicode)
     unicode_type = unicode
 
+try:
+    import win_unicode_console
+except ImportError:
+    win_unicode_console = None
+
+
 DEFAULT_ENCODING = sys.getdefaultencoding()
 
 ON_WINDOWS = (platform.system() == 'Windows')
@@ -45,6 +64,7 @@ ON_LINUX = (platform.system() == 'Linux')
 ON_ARCH = (platform.linux_distribution()[0] == 'arch')
 ON_POSIX = (os.name == 'posix')
 IS_ROOT = ctypes.windll.shell32.IsUserAnAdmin() != 0 if ON_WINDOWS else os.getuid() == 0
+HAVE_PYGMENTS = bool(pygments_version())
 
 VER_3_4 = (3, 4)
 VER_3_5 = (3, 5)
@@ -53,6 +73,7 @@ VER_FULL = sys.version_info[:3]
 VER_MAJOR_MINOR = sys.version_info[:2]
 V_MAJOR_MINOR = 'v{0}{1}'.format(*sys.version_info[:2])
 
+
 def docstring_by_version(**kwargs):
     """Sets a docstring by the python version."""
     doc = kwargs.get(V_MAJOR_MINOR, None)
@@ -68,6 +89,13 @@ class XonshError(Exception):
     pass
 
 
+class DefaultNotGivenType(object):
+    """Singleton for representing when no default value is given."""
+
+
+DefaultNotGiven = DefaultNotGivenType()
+
+
 def subproc_toks(line, mincol=-1, maxcol=None, lexer=None, returnline=False):
     """Excapsulates tokens in a source code line in a uncaptured
     subprocess $[] starting at a minimum column. If there are no tokens
@@ -102,7 +130,14 @@ def subproc_toks(line, mincol=-1, maxcol=None, lexer=None, returnline=False):
             tok.type = 'NEWLINE'
             tok.value = '\n'
             tok.lineno -= 1
-            tok.lexpos = len(line)
+            if len(toks) >= 2:
+                prev_tok_end = toks[-2].lexpos + len(toks[-2].value)
+            else:
+                prev_tok_end = len(line)
+            if '#' in line[prev_tok_end:]:
+                tok.lexpos = prev_tok_end  # prevents wrapping comments
+            else:
+                tok.lexpos = len(line)
             break
     else:
         if len(toks) == 0:
@@ -215,74 +250,6 @@ def get_sep():
             os.sep)
 
 
-TERM_COLORS = {
-    # Reset
-    'NO_COLOR': '\001\033[0m\002',  # Text Reset
-    # Regular Colors
-    'BLACK': '\001\033[0;30m\002',  # BLACK
-    'RED': '\001\033[0;31m\002',  # RED
-    'GREEN': '\001\033[0;32m\002',  # GREEN
-    'YELLOW': '\001\033[0;33m\002',  # YELLOW
-    'BLUE': '\001\033[0;34m\002',  # BLUE
-    'PURPLE': '\001\033[0;35m\002',  # PURPLE
-    'CYAN': '\001\033[0;36m\002',  # CYAN
-    'WHITE': '\001\033[0;37m\002',  # WHITE
-    # Bold
-    'BOLD_BLACK': '\001\033[1;30m\002',  # BLACK
-    'BOLD_RED': '\001\033[1;31m\002',  # RED
-    'BOLD_GREEN': '\001\033[1;32m\002',  # GREEN
-    'BOLD_YELLOW': '\001\033[1;33m\002',  # YELLOW
-    'BOLD_BLUE': '\001\033[1;34m\002',  # BLUE
-    'BOLD_PURPLE': '\001\033[1;35m\002',  # PURPLE
-    'BOLD_CYAN': '\001\033[1;36m\002',  # CYAN
-    'BOLD_WHITE': '\001\033[1;37m\002',  # WHITE
-    # Underline
-    'UNDERLINE_BLACK': '\001\033[4;30m\002',  # BLACK
-    'UNDERLINE_RED': '\001\033[4;31m\002',  # RED
-    'UNDERLINE_GREEN': '\001\033[4;32m\002',  # GREEN
-    'UNDERLINE_YELLOW': '\001\033[4;33m\002',  # YELLOW
-    'UNDERLINE_BLUE': '\001\033[4;34m\002',  # BLUE
-    'UNDERLINE_PURPLE': '\001\033[4;35m\002',  # PURPLE
-    'UNDERLINE_CYAN': '\001\033[4;36m\002',  # CYAN
-    'UNDERLINE_WHITE': '\001\033[4;37m\002',  # WHITE
-    # Background
-    'BACKGROUND_BLACK': '\001\033[40m\002',  # BLACK
-    'BACKGROUND_RED': '\001\033[41m\002',  # RED
-    'BACKGROUND_GREEN': '\001\033[42m\002',  # GREEN
-    'BACKGROUND_YELLOW': '\001\033[43m\002',  # YELLOW
-    'BACKGROUND_BLUE': '\001\033[44m\002',  # BLUE
-    'BACKGROUND_PURPLE': '\001\033[45m\002',  # PURPLE
-    'BACKGROUND_CYAN': '\001\033[46m\002',  # CYAN
-    'BACKGROUND_WHITE': '\001\033[47m\002',  # WHITE
-    # High Intensity
-    'INTENSE_BLACK': '\001\033[0;90m\002',  # BLACK
-    'INTENSE_RED': '\001\033[0;91m\002',  # RED
-    'INTENSE_GREEN': '\001\033[0;92m\002',  # GREEN
-    'INTENSE_YELLOW': '\001\033[0;93m\002',  # YELLOW
-    'INTENSE_BLUE': '\001\033[0;94m\002',  # BLUE
-    'INTENSE_PURPLE': '\001\033[0;95m\002',  # PURPLE
-    'INTENSE_CYAN': '\001\033[0;96m\002',  # CYAN
-    'INTENSE_WHITE': '\001\033[0;97m\002',  # WHITE
-    # Bold High Intensity
-    'BOLD_INTENSE_BLACK': '\001\033[1;90m\002',  # BLACK
-    'BOLD_INTENSE_RED': '\001\033[1;91m\002',  # RED
-    'BOLD_INTENSE_GREEN': '\001\033[1;92m\002',  # GREEN
-    'BOLD_INTENSE_YELLOW': '\001\033[1;93m\002',  # YELLOW
-    'BOLD_INTENSE_BLUE': '\001\033[1;94m\002',  # BLUE
-    'BOLD_INTENSE_PURPLE': '\001\033[1;95m\002',  # PURPLE
-    'BOLD_INTENSE_CYAN': '\001\033[1;96m\002',  # CYAN
-    'BOLD_INTENSE_WHITE': '\001\033[1;97m\002',  # WHITE
-    # High Intensity backgrounds
-    'BACKGROUND_INTENSE_BLACK': '\001\033[0;100m\002',  # BLACK
-    'BACKGROUND_INTENSE_RED': '\001\033[0;101m\002',  # RED
-    'BACKGROUND_INTENSE_GREEN': '\001\033[0;102m\002',  # GREEN
-    'BACKGROUND_INTENSE_YELLOW': '\001\033[0;103m\002',  # YELLOW
-    'BACKGROUND_INTENSE_BLUE': '\001\033[0;104m\002',  # BLUE
-    'BACKGROUND_INTENSE_PURPLE': '\001\033[0;105m\002',  # PURPLE
-    'BACKGROUND_INTENSE_CYAN': '\001\033[0;106m\002',  # CYAN
-    'BACKGROUND_INTENSE_WHITE': '\001\033[0;107m\002',  # WHITE
-}
-
 
 def fallback(cond, backup):
     """Decorator for returning the object if cond is true and a backup if cond is false.
@@ -634,6 +601,16 @@ def to_completions_display_value(x):
     return x
 
 
+def setup_win_unicode_console(enable):
+    """"Enables or disables unicode display on windows."""
+    enable = to_bool(enable)
+    if ON_WINDOWS and win_unicode_console:
+        if enable:
+            win_unicode_console.enable()
+        else:
+            win_unicode_console.disable()
+    return enable
+
 # history validation
 
 _min_to_sec = lambda x: 60.0 * float(x)
@@ -729,133 +706,33 @@ def history_tuple_to_str(x):
     """Converts a valid history tuple to a canonical string."""
     return '{0} {1}'.format(*x)
 
-#
-# prompt toolkit tools
-#
 
-
-class FakeChar(str):
-    """Class that holds a single char and escape sequences that surround it.
-
-    It is used as a workaround for the fact that prompt_toolkit doesn't display
-    colorful prompts correctly.
-    It behaves like normal string created with prefix + char + suffix, but has
-    two differences:
-
-    * len() always returns 2
-
-    * iterating over instance of this class is the same as iterating over
-      the single char - prefix and suffix are ommited.
+def format_color(string, **kwargs):
+    """Formats strings that may contain colors. This simply dispatches to the
+    shell instances method of the same name. The results of this function should
+    be directly usable by print_color().
     """
-    def __new__(cls, char, prefix='', suffix=''):
-        return str.__new__(cls, prefix + char + suffix)
-
-    def __init__(self, char, prefix='', suffix=''):
-        self.char = char
-        self.prefix = prefix
-        self.suffix = suffix
-        self.length = 2
-        self.iterated = False
-
-    def __len__(self):
-        return self.length
-
-    def __iter__(self):
-        return iter(self.char)
+    return builtins.__xonsh_shell__.shell.format_color(string, **kwargs)
 
 
-RE_HIDDEN_MAX = re.compile('(\001.*?\002)+')
-
-
-_PT_COLORS_DARK = {'BLACK': '#000000',
-                   'RED': '#ff1010',
-                   'GREEN': '#00FF18',
-                   'YELLOW': '#FFFF00',
-                   'BLUE': '#0000D2',
-                   'PURPLE': '#FF00FF',
-                   'CYAN': '#00FFFF',
-                   'WHITE': '#FFFFFF',
-                   'GRAY': '#c0c0c0'}
-
-_PT_COLORS_LIGHT = {'BLACK': '#000000',
-                    'RED': '#800000',
-                    'GREEN': '#008000',
-                    'YELLOW': '#808000',
-                    'BLUE': '#000080',
-                    'PURPLE': '#800080',
-                    'CYAN': '#008080',
-                    'WHITE': '#FFFFFF',
-                    'GRAY': '#008080'}
-
-_PT_STYLE = {'BOLD': 'bold',
-             'UNDERLINE': 'underline',
-             'INTENSE': 'italic'}
-
-
-def _make_style(color_name):
-    """ Convert color names to pygments styles codes """
-    style = []
-    for k, v in _PT_STYLE.items():
-        if k in color_name:
-            style.append(v)
-    _custom_colors = builtins.__xonsh_env__.get('PROMPT_TOOLKIT_COLORS')
-    for k, v in _custom_colors.items():
-        if k in color_name:
-            style.append(v)
-    for k, v in _PT_COLORS_DARK.items():
-        if k not in _custom_colors and k in color_name:
-            style.append(v)
-    return ' '.join(style)
-
-
-def get_xonsh_color_names(color_code):
-    """ Makes a reverse lookup in TERM_COLORS  """
-    try:
-        return next(k for k, v in TERM_COLORS.items() if v == color_code)
-    except StopIteration:
-        return 'NO_COLOR'
-
-
-def format_prompt_for_prompt_toolkit(prompt):
-    """Converts a prompt with color codes to a pygments style and tokens
+def print_color(string, **kwargs):
+    """Prints a string that may contain colors. This dispatched to the shell
+    method of the same name. Colors will be formatted if they have not already
+    been.
     """
-    parts = RE_HIDDEN_MAX.split(prompt)
-    # ensure that parts is [colorcode, string, colorcode, string,...]
-    if parts and len(parts[0]) == 0:
-        parts = parts[1:]
-    else:
-        parts.insert(0, '')
-    if len(parts) % 2 != 0:
-        parts.append()
-    strings = parts[1::2]
-    token_names = [get_xonsh_color_names(c) for c in parts[::2]]
-    cstyles = [_make_style(c) for c in token_names]
-    return token_names, cstyles, strings
+    builtins.__xonsh_shell__.shell.print_color(string, **kwargs)
 
 
-def format_color(string):
-    """Formats strings that contain xonsh.tools.TERM_COLORS values."""
-    s = string.format(**TERM_COLORS).replace('\001', '').replace('\002', '')
-    return s
+def color_style_names():
+    """Returns an iterable of all available style names."""
+    return builtins.__xonsh_shell__.shell.color_style_names()
 
 
-def print_color(string, file=sys.stdout):
-    """Print strings that contain xonsh.tools.TERM_COLORS values. By default
-    `sys.stdout` is used as the output stream but an alternate can be specified
-    by the `file` keyword argument."""
-    print(format_color(string), file=file)
+def color_style():
+    """Returns the current color map."""
+    return builtins.__xonsh_shell__.shell.color_style()
 
 
-def escape_color(string):
-    """Escapes color formatting, ie '{RED}' becomes '{{RED}}'."""
-    s = string
-    for color in TERM_COLORS.keys():
-        if color in s:
-            bc = '{' + color + '}'  # braced color
-            dbc = '{' + bc + '}'  # double-braced color
-            s = s.replace(bc, dbc)
-    return s
-
 
 _RE_STRING_START = "[bBrRuU]*"
 _RE_STRING_TRIPLE_DOUBLE = '"""'
@@ -1067,7 +944,7 @@ def expandvars(path):
 
 #
 # File handling tools
-# 
+#
 
 def backup_file(fname):
     """Moves an existing file to a new name that has the current time right
@@ -1079,3 +956,55 @@ def backup_file(fname):
     base, ext = os.path.splitext(fname)
     newfname = base + '.' + datetime.now().isoformat() + ext
     shutil.move(fname, newfname)
+
+
+def normabspath(p):
+    """Retuns as normalized absolute path, namely, normcase(abspath(p))"""
+    return os.path.normcase(os.path.abspath(p))
+
+
+class CommandsCache(Set):
+    """A lazy cache representing the commands available on the file system."""
+
+    def __init__(self):
+        self._cmds_cache = frozenset()
+        self._path_checksum = None
+        self._alias_checksum = None
+        self._path_mtime = -1
+
+    def __contains__(self, item):
+        return item in self.all_commands
+
+    def __iter__(self):
+        return iter(self.all_commands)
+
+    def __len__(self):
+        return len(self.all_commands)
+
+    @property
+    def all_commands(self):
+        path = builtins.__xonsh_env__.get('PATH', [])
+        # did PATH change?
+        path_hash = hash(frozenset(path))
+        cache_valid = path_hash == self._path_checksum
+        self._path_checksum = path_hash
+        # did aliases change?
+        al_hash = hash(frozenset(builtins.aliases.keys()))
+        cache_valid = cache_valid and al_hash == self._alias_checksum
+        self._alias_checksum = al_hash
+        pm = self._path_mtime
+        # did the contents of any directory in PATH change?
+        for d in filter(os.path.isdir, path):
+            m = os.stat(d).st_mtime
+            if m > pm:
+                pm = m
+                cache_valid = False
+        self._path_mtime = pm
+        if cache_valid:
+            return self._cmds_cache
+        allcmds = set()
+        for d in filter(os.path.isdir, path):
+            allcmds |= set(os.listdir(d))
+        allcmds |= set(builtins.aliases.keys())
+        self._cmds_cache = frozenset(allcmds)
+        return self._cmds_cache
