@@ -2,9 +2,11 @@
 import os
 import ast
 import json
+import shutil
 import textwrap
-import  builtins
+import builtins
 import functools
+import itertools
 from pprint import pformat
 from argparse import ArgumentParser
 
@@ -19,12 +21,12 @@ from xonsh.wizard import (Wizard, Pass, Message, Save, Load, YesNo, Input,
     PromptVisitor, While, StoreNonEmpty, create_truefalse_cond, YN)
 
 
-HR = "'`-.,¸,.-*¯`-.,¸,.-*¯`-.,¸,.-*¯`-.,¸,.-*¯`-.,¸,.-*¯`-.,¸,.-*¯`-.,¸,.-*'"
+HR = "'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'"
 WIZARD_HEAD = """
           {{BOLD_WHITE}}Welcome to the xonsh configuration wizard!{{NO_COLOR}}
           {{YELLOW}}------------------------------------------{{NO_COLOR}}
-This will present a guided tour through setting up the xonsh static 
-config file. Xonsh will automatically ask you if you want to run this 
+This will present a guided tour through setting up the xonsh static
+config file. Xonsh will automatically ask you if you want to run this
 wizard if the configuration file does not exist. However, you can
 always rerun this wizard with the xonfig command:
 
@@ -48,12 +50,12 @@ WIZARD_FS = """
                       {{BOLD_WHITE}}Foreign Shell Setup{{NO_COLOR}}
                       {{YELLOW}}-------------------{{NO_COLOR}}
 The xonsh shell has the ability to interface with foreign shells such
-as Bash, zsh, or fish. 
+as Bash, zsh, or fish.
 
-For configuration, this means that xonsh can load the environment, 
-aliases, and functions specified in the config files of these shells. 
-Naturally, these shells must be available on the system to work. 
-Being able to share configuration (and source) from foreign shells 
+For configuration, this means that xonsh can load the environment,
+aliases, and functions specified in the config files of these shells.
+Naturally, these shells must be available on the system to work.
+Being able to share configuration (and source) from foreign shells
 makes it easier to transition to and from xonsh.
 """.format(hr=HR)
 
@@ -69,18 +71,21 @@ Still, setting environment variables in this way can help define
 options that are global to the system or user.
 
 The following lists the environment variable name, its documentation,
-the default value, and the current value. The default and current 
+the default value, and the current value. The default and current
 values are presented as pretty repr strings of their Python types.
 
-{{BOLD_GREEN}}Note:{{NO_COLOR}} Simply hitting enter for any environment variable 
+{{BOLD_GREEN}}Note:{{NO_COLOR}} Simply hitting enter for any environment variable
 will accept the default value for that entry.
+""".format(hr=HR)
+
+WIZARD_ENV_QUESTION = "Would you like to set env vars now, " + YN
 
-Would you like to set env vars now, """.format(hr=HR) + YN
 
 WIZARD_TAIL = """
 Thanks for using the xonsh configuration wizard!"""
 
 
+
 def make_fs():
     """Makes the foreign shell part of the wizard."""
     cond = create_truefalse_cond(prompt='Add a foreign shell, ' + YN)
@@ -131,12 +136,13 @@ def _wrap_paragraphs(text, width=70, **kwargs):
     s = '\n'.join(pars)
     return s
 
-ENVVAR_PROMPT = """
+ENVVAR_MESSAGE = """
 {{BOLD_CYAN}}${name}{{NO_COLOR}}
 {docstr}
 {{RED}}default value:{{NO_COLOR}} {default}
-{{RED}}current value:{{NO_COLOR}} {current}
-{{BOLD_GREEN}}>>>{{NO_COLOR}} """
+{{RED}}current value:{{NO_COLOR}} {current}"""
+
+ENVVAR_PROMPT = "{BOLD_GREEN}>>>{NO_COLOR} "
 
 def make_envvar(name):
     """Makes a StoreNonEmpty node for an environment variable."""
@@ -153,20 +159,25 @@ def make_envvar(name):
     curr = pformat(curr, width=69)
     if '\n' in curr:
         curr = '\n' + curr
-    prompt = ENVVAR_PROMPT.format(name=name, default=default, current=curr,
+    msg = ENVVAR_MESSAGE.format(name=name, default=default, current=curr,
                                 docstr=_wrap_paragraphs(vd.docstr, width=69))
+    mnode = Message(message=msg)
     ens = env.get_ensurer(name)
     path = '/env/' + name
-    node = StoreNonEmpty(prompt, converter=ens.convert, show_conversion=True,
-                         path=path)
-    return node
-    
+    pnode = StoreNonEmpty(ENVVAR_PROMPT, converter=ens.convert,
+                          show_conversion=True, path=path)
+    return mnode, pnode
+
 
 def make_env():
     """Makes an environment variable wizard."""
     kids = map(make_envvar, sorted(builtins.__xonsh_env__.docs.keys()))
-    kids = [k for k in kids if k is not None]
-    wiz = Wizard(children=kids)
+    flatkids = []
+    for k in kids:
+        if k is None:
+            continue
+        flatkids.extend(k)
+    wiz = Wizard(children=flatkids)
     return wiz
 
 
@@ -185,7 +196,8 @@ def make_wizard(default_file=None, confirm=False):
             Load(default_file=default_file, check=True),
             Message(message=WIZARD_FS),
             make_fs(),
-            YesNo(question=WIZARD_ENV, yes=make_env(), no=Pass()),
+            Message(message=WIZARD_ENV),
+            YesNo(question=WIZARD_ENV_QUESTION, yes=make_env(), no=Pass()),
             Message(message='\n' + HR + '\n'),
             Save(default_file=default_file, check=True),
             Message(message=WIZARD_TAIL),
@@ -206,7 +218,7 @@ def _wizard(ns):
         try:
             pv.visit()
         except (KeyboardInterrupt, Exception):
-            pass
+            tools.print_exception()
 
 
 def _format_human(data):
@@ -227,16 +239,17 @@ def _format_json(data):
     data = {k.replace(' ', '_'): v for k, v in data}
     s = json.dumps(data, sort_keys=True, indent=1) + '\n'
     return s
-    
+
 
 def _info(ns):
     data = [
-        ('xonsh', XONSH_VERSION), 
+        ('xonsh', XONSH_VERSION),
         ('Python', '.'.join(map(str, tools.VER_FULL))),
         ('PLY', ply.__version__),
         ('have readline', is_readline_available()),
         ('prompt toolkit', prompt_toolkit_version() if \
                            is_prompt_toolkit_available() else False),
+        ('pygments', tools.pygments_version()),
         ('on posix', tools.ON_POSIX),
         ('on linux', tools.ON_LINUX),
         ('on arch', tools.ON_ARCH),
@@ -250,32 +263,109 @@ def _info(ns):
     return s
 
 
+def _styles(ns):
+    env = builtins.__xonsh_env__
+    curr = env.get('XONSH_COLOR_STYLE')
+    styles = sorted(tools.color_style_names())
+    if ns.json:
+        s = json.dumps(styles, sort_keys=True, indent=1)
+        print(s)
+        return
+    lines = []
+    for style in styles:
+        if style == curr:
+            lines.append('* {GREEN}' + style + '{NO_COLOR}')
+        else:
+            lines.append('  ' + style)
+    s = '\n'.join(lines)
+    tools.print_color(s)
+
+
+def _str_colors(cmap, cols):
+    color_names = sorted(cmap.keys(), key=(lambda s: (len(s), s)))
+    grper = lambda s: min(cols // (len(s) + 1), 8)
+    lines = []
+    for n, group in itertools.groupby(color_names, key=grper):
+        width = cols // n
+        line = ''
+        for i, name in enumerate(group):
+            buf = ' ' * (width - len(name))
+            line += '{' + name + '}' + name + '{NO_COLOR}' + buf
+            if (i+1)%n == 0:
+                lines.append(line)
+                line = ''
+        if len(line) != 0:
+            lines.append(line)
+    return '\n'.join(lines)
+
+def _tok_colors(cmap, cols):
+    from xonsh.pyghooks import Color
+    nc = Color.NO_COLOR
+    names_toks = {}
+    for t in cmap.keys():
+        name = str(t)
+        if name.startswith('Token.Color.'):
+            _, _, name = name.rpartition('.')
+        names_toks[name] = t
+    color_names = sorted(names_toks.keys(), key=(lambda s: (len(s), s)))
+    grper = lambda s: min(cols // (len(s) + 1), 8)
+    toks = []
+    for n, group in itertools.groupby(color_names, key=grper):
+        width = cols // n
+        for i, name in enumerate(group):
+            toks.append((names_toks[name], name))
+            buf = ' ' * (width - len(name))
+            if (i+1)%n == 0:
+                buf += '\n'
+            toks.append((nc, buf))
+        if not toks[-1][1].endswith('\n'):
+            toks[-1] = (nc, toks[-1][1] + '\n')
+    return toks
+
+def _colors(ns):
+    cols, _ = shutil.get_terminal_size()
+    cmap = tools.color_style()
+    akey = next(iter(cmap))
+    if isinstance(akey, str):
+        s = _str_colors(cmap, cols)
+    else:
+        s = _tok_colors(cmap, cols)
+    tools.print_color(s)
+
+
 @functools.lru_cache()
 def _create_parser():
-    p = ArgumentParser(prog='xonfig', 
+    p = ArgumentParser(prog='xonfig',
                        description='Manages xonsh configuration.')
     subp = p.add_subparsers(title='action', dest='action')
     info = subp.add_parser('info', help=('displays configuration information, '
                                          'default action'))
-    info.add_argument('--json', action='store_true', default=False, 
+    info.add_argument('--json', action='store_true', default=False,
                       help='reports results as json')
     wiz = subp.add_parser('wizard', help=('displays configuration information, '
                                          'default action'))
-    wiz.add_argument('--file', default=None, 
+    wiz.add_argument('--file', default=None,
                      help='config file location, default=$XONSHCONFIG')
-    wiz.add_argument('--confirm', action='store_true', default=False, 
+    wiz.add_argument('--confirm', action='store_true', default=False,
                       help='confirm that the wizard should be run.')
+    sty = subp.add_parser('styles', help='prints available xonsh color styles')
+    sty.add_argument('--json', action='store_true', default=False,
+                     help='reports results as json')
+    clrs = subp.add_parser('colors', help=('displays the color palette for '
+                                           'the current xonsh color style'))
     return p
 
 
 _MAIN_ACTIONS = {
     'info': _info,
     'wizard': _wizard,
+    'styles': _styles,
+    'colors': _colors,
     }
 
 def main(args=None):
     """Main xonfig entry point."""
-    if not args or (args[0] not in _MAIN_ACTIONS and 
+    if not args or (args[0] not in _MAIN_ACTIONS and
                     args[0] not in {'-h', '--help'}):
         args.insert(0, 'info')
     parser = _create_parser()
